@@ -1,8 +1,8 @@
 import argparse
+import asyncio
 import logging
 import os
 from typing import List, Optional, Text, Dict, Union, Any
-import asyncio
 
 from rasa.cli import SubParsersAction
 import rasa.shared.data
@@ -22,7 +22,6 @@ from rasa.shared.constants import (
     DEFAULT_MODELS_PATH,
     DEFAULT_DATA_PATH,
     DEFAULT_RESULTS_PATH,
-    DEFAULT_DOMAIN_PATH,
 )
 import rasa.shared.utils.validation as validation_utils
 import rasa.cli.utils
@@ -156,6 +155,7 @@ async def run_nlu_test_async(
     percentages: List[int],
     runs: int,
     no_errors: bool,
+    domain_path: Text,
     all_args: Dict[Text, Any],
 ) -> None:
     """Runs NLU tests.
@@ -172,6 +172,7 @@ async def run_nlu_test_async(
                           or not.
         percentages: defines the exclusion percentage of the training data.
         runs: number of comparison runs to make.
+        domain_path: path to domain.
         no_errors: indicates if incorrect predictions should be written to a file
                    or not.
     """
@@ -183,7 +184,7 @@ async def run_nlu_test_async(
 
     data_path = rasa.cli.utils.get_validated_path(data_path, "nlu", DEFAULT_DATA_PATH)
     test_data_importer = TrainingDataImporter.load_from_dict(
-        training_data_paths=[data_path], domain_path=DEFAULT_DOMAIN_PATH
+        training_data_paths=[data_path], domain_path=domain_path
     )
     nlu_data = test_data_importer.get_nlu_data()
 
@@ -234,7 +235,7 @@ async def run_nlu_test_async(
             models_path, "model", DEFAULT_MODELS_PATH
         )
 
-        await test_nlu(model_path, data_path, output, all_args)
+        await test_nlu(model_path, data_path, output, all_args, domain_path=domain_path)
 
 
 def run_nlu_test(args: argparse.Namespace) -> None:
@@ -253,6 +254,7 @@ def run_nlu_test(args: argparse.Namespace) -> None:
             args.percentages,
             args.runs,
             args.no_errors,
+            args.domain,
             vars(args),
         )
     )
