@@ -39,9 +39,9 @@ class SingleStateFeaturizer:
 
     def __init__(self) -> None:
         """Initialize the single state featurizer."""
-        self._default_feature_states = {}
-        self.action_texts = []
-        self.entity_tag_specs = []
+        self._default_feature_states: Dict[Text, Any] = {}
+        self.action_texts: List[Text] = []
+        self.entity_tag_specs: List[EntityTagSpec] = []
 
     def _create_entity_tag_specs(
         self, bilou_tagging: bool = False
@@ -309,9 +309,11 @@ class SingleStateFeaturizer:
         ):
             # we cannot build a classifier with fewer than 2 classes
             return {}
-
-        message = precomputations.lookup_message(user_text=entity_data[TEXT])
-        message.data[ENTITIES] = entity_data[ENTITIES]
+        if precomputations is None:
+            message = None
+        else:
+            message = precomputations.lookup_message(user_text=entity_data[TEXT])
+            message.data[ENTITIES] = entity_data[ENTITIES]
 
         if not message:
             return {}
