@@ -105,13 +105,16 @@ class TrackerFeaturizer:
         Returns:
             Featurized tracker states.
         """
-        return [
-            [
-                self.state_featurizer.encode_state(state, precomputations)
-                for state in tracker_states
+        if self.state_featurizer is None:
+            return [[{}]]
+        else:
+            return [
+                [
+                    self.state_featurizer.encode_state(state, precomputations)
+                    for state in tracker_states
+                ]
+                for tracker_states in trackers_as_states
             ]
-            for tracker_states in trackers_as_states
-        ]
 
     @staticmethod
     def _convert_labels_to_ids(
@@ -152,15 +155,18 @@ class TrackerFeaturizer:
         Returns:
             Trackers as entity features.
         """
-        return [
-            [
-                self.state_featurizer.encode_entities(
-                    entity_data, precomputations, bilou_tagging
-                )
-                for entity_data in trackers_entities
+        if self.state_featurizer is None:
+            return [[{}]]
+        else:
+            return [
+                [
+                    self.state_featurizer.encode_entities(
+                        entity_data, precomputations, bilou_tagging
+                    )
+                    for entity_data in trackers_entities
+                ]
+                for trackers_entities in trackers_as_entities
             ]
-            for trackers_entities in trackers_as_entities
-        ]
 
     @staticmethod
     def _entity_data(event: UserUttered) -> Dict[Text, Any]:
@@ -502,9 +508,7 @@ class FullDialogueTrackerFeaturizer(TrackerFeaturizer):
         domain: Domain,
         omit_unset_slots: bool = False,
         ignore_action_unlikely_intent: bool = False,
-    ) -> Tuple[
-        List[List[State]], List[List[Optional[Text]]], List[List[Dict[Text, Any]]]
-    ]:
+    ) -> Tuple[List[List[State]], List[List[Text]], List[List[Dict[Text, Any]]]]:
         """Transforms trackers to states, action labels, and entity data.
 
         Args:
@@ -555,7 +559,9 @@ class FullDialogueTrackerFeaturizer(TrackerFeaturizer):
                 if not event.unpredictable:
                     # only actions which can be
                     # predicted at a stories start
-                    actions.append(event.action_name or event.action_text)
+                    action = event.action_name or event.action_text
+                    if action is not None:
+                        actions.append(action)
                     entities.append(entity_data)
                 else:
                     # unpredictable actions can be
@@ -1056,7 +1062,7 @@ class IntentMaxHistoryTrackerFeaturizer(MaxHistoryTrackerFeaturizer):
                     tracker_states[:label_index], self.max_history
                 )
                 label = [event.intent_name or event.text]
-                entities = [{}]
+                entities: List[Dict[Text, Any]] = [{}]
 
                 yield sliced_states, label, entities
 
