@@ -1,3 +1,4 @@
+import copy
 import logging
 import os
 from typing import Text, Dict, Optional, List, Any, Iterable, Tuple, Union
@@ -13,13 +14,20 @@ from rasa.constants import RESULTS_FILE, NUMBER_OF_TRAINING_STORIES_FILE
 from rasa.exceptions import ModelNotFound
 from rasa.shared.constants import DEFAULT_RESULTS_PATH
 import rasa.shared.nlu.training_data.loading
-from rasa.shared.importers.autoconfig import TrainingType
+from rasa.shared.data import TrainingType
 from rasa.shared.nlu.training_data.training_data import TrainingData
 import rasa.model
 
 logger = logging.getLogger(__name__)
 
 
+class ClassificationReportException(Exception):
+    """Raised when clf_report doesn't correctly set accuracy and/or micro avg.
+
+    sklearn.metrics.classification_report should provide either accuracy or micro avg.
+    """
+
+
 async def test_core_models_in_directory(
     model_directory: Text,
     stories: Text,
@@ -183,6 +191,7 @@ async def test_nlu(
     nlu_data: Optional[Text],
     output_directory: Text = DEFAULT_RESULTS_PATH,
     additional_arguments: Optional[Dict] = None,
+    domain_path: Optional[Text] = None,
 ) -> None:
     """Tests the NLU Model."""
     from rasa.nlu.test import run_evaluation
@@ -206,7 +215,11 @@ async def test_nlu(
         )
         _agent = Agent.load(model_path=model)
         await run_evaluation(
-            nlu_data, _agent.processor, output_directory=output_directory, **kwargs
+            nlu_data,
+            _agent.processor,
+            output_directory=output_directory,
+            domain_path=domain_path,
+            **kwargs,
         )
     else:
         rasa.shared.utils.cli.print_error(
@@ -236,7 +249,7 @@ async def compare_nlu_models(
     bases = [os.path.basename(nlu_config) for nlu_config in configs]
     model_names = [os.path.splitext(base)[0] for base in bases]
 
-    f1_score_results = {
+    f1_score_results: Dict[Text, List[List[float]]] = {
         model_name: [[] for _ in range(runs)] for model_name in model_names
     }
 
@@ -362,9 +375,50 @@ def get_evaluation_metrics(
     f1 = metrics.f1_score(targets, predictions, labels=labels, average="weighted")
     accuracy = metrics.accuracy_score(targets, predictions)
 
+    if output_dict:
+        report = make_classification_report_complete(report, accuracy)
+
     return report, precision, f1, accuracy
 
 
+def make_classification_report_complete(report: dict, accuracy: float) -> dict:
+    """Completes the sklearn classification report with accuracy xor micro avg.
+
+    Args:
+        report: Report generated by metrics.classification_report with output_dict=True
+        accuracy: Model accuracy
+
+    Raises:
+        Exception: When sklearn.metrics.classification_report
+            behaves different to our expectation.
+
+    Returns:
+        report: Report generated by metrics.classification_report
+            enhanced with accuracy xor micro avg.
+    """
+    report = copy.deepcopy(report)
+    if "accuracy" in report and "micro avg" not in report:
+        # micro avg corresponds to accuracy in this case
+        # and is the same for all metrics
+        acc = report["accuracy"]
+        support = report["macro avg"]["support"]
+        report["micro avg"] = {
+            "precision": acc,
+            "recall": acc,
+            "f1-score": acc,
+            "support": support,
+        }
+    elif "accuracy" not in report and "micro avg" in report:
+        # Due to provided labels, micro avg can have recall != precision
+        # The accuracy therefore has to be inferred separately
+        report["accuracy"] = accuracy
+    else:
+        raise ClassificationReportException(
+            "This cannot happen according to classification_report's docs"
+        )
+    return report
+
+
 def clean_labels(labels: Iterable[Text]) -> List[Text]:
     """Remove `None` labels. sklearn metrics do not support them.
 
