@@ -68,7 +68,9 @@ class RasaYAMLReader(TrainingDataReader):
             e.filename = self.filename
             raise e
 
-    def reads(self, string: Text, **kwargs: Any) -> "TrainingData":
+    def reads(  # type: ignore[override]
+        self, string: Text, **kwargs: Any
+    ) -> "TrainingData":
         """Reads TrainingData in YAML format from a string.
 
         Args:
@@ -429,7 +431,8 @@ class RasaYAMLWriter(TrainingDataWriter):
 
     @classmethod
     def process_synonyms(cls, training_data: "TrainingData") -> List[OrderedDict]:
-        inverted_synonyms = OrderedDict()
+        """Serializes the synonyms."""
+        inverted_synonyms: Dict[Text, List[Dict]] = OrderedDict()
         for example, synonym in training_data.entity_synonyms.items():
             if not inverted_synonyms.get(synonym):
                 inverted_synonyms[synonym] = []
@@ -444,7 +447,8 @@ class RasaYAMLWriter(TrainingDataWriter):
 
     @classmethod
     def process_regexes(cls, training_data: "TrainingData") -> List[OrderedDict]:
-        inverted_regexes = OrderedDict()
+        """Serializes the regexes."""
+        inverted_regexes: Dict[Text, List[Text]] = OrderedDict()
         for regex in training_data.regex_features:
             if not inverted_regexes.get(regex["name"]):
                 inverted_regexes[regex["name"]] = []
